"""In-process tests for the flatpak generator"""

import logging
import os
import pytest
import shutil
import tempfile
import yaml

from click.testing import CliRunner

from gi.repository import Modulemd

from _fedmod.cli import _cli_commands
from _fedmod.flatpak_generator import FlatpakGenerator

log = logging.getLogger(__name__)

def _generate_flatpak(rpm):
    cmd = ['rpm2flatpak']
    cmd.append(rpm)

    workdir = tempfile.mkdtemp()
    try:
        os.chdir(workdir)
        runner = CliRunner()
        result = runner.invoke(_cli_commands, cmd)
        assert result.exit_code == 0

        modulemd_fname = rpm + '.yaml'
        with open(rpm + '.yaml') as f:
            contents = f.read()

        log.info('%s:\n%s\n', modulemd_fname, contents)

        objs = Modulemd.objects_from_string(contents)
        assert len(objs) == 1
        assert isinstance(objs[0], Modulemd.Module)
        modmd = objs[0]

        with open('container.yaml') as f:
            contents = f.read()

        log.info('container.yaml:\n%s\n',contents)
        container_yaml = yaml.safe_load(contents)
    finally:
        shutil.rmtree(workdir)

    return modmd, container_yaml


class TestFlatpak(object):

    def test_generated_flatpak_files(self):
        modmd, container_yaml = _generate_flatpak('eog')

        # Expected description for 'grep'
        assert modmd.props.summary == "Eye of GNOME image viewer"
        assert modmd.props.description.startswith("The Eye of GNOME image viewer (eog) is")

        # Expected licenses for 'grep'
        assert modmd.props.module_licenses.get() == ['MIT']
        assert modmd.props.content_licenses.get() == []

        # Only given modules are listed in the public API
        assert sorted(modmd.props.rpm_api.get()) == ['eog']

        # Expected components
        assert set(modmd.props.components_rpm) == set([
            'bubblewrap', 'eog', 'exempi', 'gnome-desktop3', 'libpeas'
        ])

        # Expected module dependencies for grep
        dependencies = modmd.props.dependencies
        assert len(dependencies) == 1

        buildrequires = dependencies[0].props.buildrequires
        assert set(buildrequires) == {'flatpak-runtime',}
        assert buildrequires['flatpak-runtime'].get() == ['f28']

        requires = dependencies[0].props.requires
        assert set(requires) == {'flatpak-runtime',}
        assert requires['flatpak-runtime'].get() == ['f28']
